==> nekoconf-0.1.4.tar.gz: nekoconf-0.1.4/tests/test_web_server.py <==
"""Tests for the web server module."""

from unittest.mock import AsyncMock, patch

import pytest
from fastapi import WebSocket
from fastapi.websockets import WebSocketDisconnect

from nekoconf.server import NekoConf, WebSocketManager


class TestWebSocketManager:
    @pytest.mark.asyncio
    async def test_websocket_lifecycle(self):
        """Test the complete lifecycle of WebSocket connections."""
        manager = WebSocketManager()

        # Create WebSockets
        websocket1 = AsyncMock(spec=WebSocket)
        websocket2 = AsyncMock(spec=WebSocket)

        # Connect them
        await manager.connect(websocket1)
        await manager.connect(websocket2)

        assert len(manager.active_connections) == 2
        assert websocket1 in manager.active_connections
        assert websocket2 in manager.active_connections

        # Broadcast
        test_message = {"type": "test"}
        await manager.broadcast(test_message)

        # Both should receive the message
        websocket1.send_json.assert_called_once_with(test_message)
        websocket2.send_json.assert_called_once_with(test_message)

        # Reset mocks for next test
        websocket1.send_json.reset_mock()
        websocket2.send_json.reset_mock()

        # Disconnect one
        manager.disconnect(websocket1)

        assert len(manager.active_connections) == 1
        assert websocket1 not in manager.active_connections
        assert websocket2 in manager.active_connections

        # Broadcast again
        await manager.broadcast(test_message)

        # Only second websocket should receive
        websocket1.send_json.assert_not_called()
        websocket2.send_json.assert_called_once()

    @pytest.mark.asyncio
    async def test_broadcast_with_failed_send(self):
        """Test broadcasting with a failed send that should disconnect the client."""
        manager = WebSocketManager()

        # Create WebSockets - one that will fail on send
        websocket1 = AsyncMock(spec=WebSocket)
        websocket2 = AsyncMock(spec=WebSocket)
        websocket2.send_json.side_effect = WebSocketDisconnect()

        # Connect them
        await manager.connect(websocket1)
        await manager.connect(websocket2)

        # Broadcast should handle the exception and disconnect the failing client
        await manager.broadcast({"type": "test"})

        # Second websocket should be disconnected
        assert len(manager.active_connections) == 1
        assert websocket1 in manager.active_connections
        assert websocket2 not in manager.active_connections


class TestNekoConf:
    """Tests for the NekoConf class."""

    def test_init(self, config_manager):
        """Test initializing the NekoConf."""
        server = NekoConf(config_manager)

        assert server.config_manager == config_manager

        assert hasattr(server, "app")
        # Updated to reflect actual implementation:
        assert hasattr(server, "ws_manager")

    def test_api_get_endpoints(self, test_client, sample_config):
        """Test the GET endpoints for configuration."""
        # Test full config
        response = test_client.get("/api/config")
        assert response.status_code == 200
        assert response.json() == sample_config

        # Test section
        response = test_client.get("/api/config/server")
        assert response.status_code == 200
        assert response.json() == sample_config["server"]

        # Test specific value
        response = test_client.get("/api/config/server/host")
        assert response.status_code == 200
        assert response.json() == sample_config["server"]["host"]

        # Test nonexistent key
        response = test_client.get("/api/config/nonexistent")
        assert response.status_code == 404
        # The exact error format depends on implementation
        assert response.json().get("detail") or response.json().get("error")

    def test_api_set_and_update(self, test_client, config_manager):
        """Test setting and updating config values."""
        # Test setting a single value
        response = test_client.post("/api/config/server/host", json={"value": "127.0.0.1"})
        assert response.status_code == 200
        assert config_manager.get("server.host") == "127.0.0.1"

        # Test bad request - exact behavior depends on implementation
        response = test_client.post("/api/config/server/host", json="not-a-json")
        assert response.status_code in [400, 422]  # FastAPI validation might return 422

        # Test updating multiple values
        update_data = {"server": {"host": "example.com", "port": 9000}}
        response = test_client.post("/api/config", json=update_data)
        assert response.status_code == 200
        # The success key might not be present in all implementations
        assert config_manager.get("server.host") == "example.com"
        assert config_manager.get("server.port") == 9000

    def test_api_delete_and_reload(self, test_client, config_manager):
        """Test deleting values and reloading config."""
        # Test delete
        response = test_client.delete("/api/config/server/debug")
        assert response.status_code == 200
        # Response format depends on implementation
        assert config_manager.get("server.debug") is None

        # Test delete nonexistent key
        response = test_client.delete("/api/config/nonexistent")
        assert response.status_code == 404
        # The error format depends on implementation

        # Test reload
        # Change in-memory config
        config_manager.config_data["server"]["host"] = "changed_value"

        # Reload via API
        response = test_client.post("/api/config/reload")
        assert response.status_code == 200

        # reload resets to the file value
        assert config_manager.get("server.host") == "localhost"

    @patch("uvicorn.run")
    def test_run(self, mock_run, web_server):
        """Test the run method."""
        web_server.run(host="127.0.0.1", port=9000, reload=True)
        mock_run.assert_called_once()
        args, kwargs = mock_run.call_args
        assert kwargs["host"] == "127.0.0.1"
        assert kwargs["port"] == 9000
        assert kwargs["reload"] is True

    # Skip the websocket tests that rely on specific implementation details
    # which may vary across versions of the application

==> nekoconf-0.1.7.tar.gz: nekoconf-0.1.7/tests/test_web_server.py <==
"""Tests for the web server module."""

from unittest.mock import AsyncMock, patch

import pytest
from fastapi import WebSocket
from fastapi.websockets import WebSocketDisconnect

from nekoconf.server.app import NekoConfigServer, NekoWsNotifier


class TestWebSocketManager:
    @pytest.mark.asyncio
    async def test_websocket_lifecycle(self):
        """Test the complete lifecycle of WebSocket connections."""
        manager = NekoWsNotifier()

        # Create WebSockets
        websocket1 = AsyncMock(spec=WebSocket)
        websocket2 = AsyncMock(spec=WebSocket)

        # Connect them
        await manager.connect(websocket1)
        await manager.connect(websocket2)

        assert len(manager.active_connections) == 2
        assert websocket1 in manager.active_connections
        assert websocket2 in manager.active_connections

        # Broadcast
        test_message = {"type": "test"}
        await manager.broadcast(test_message)

        # Both should receive the message
        websocket1.send_json.assert_called_once_with(test_message)
        websocket2.send_json.assert_called_once_with(test_message)

        # Reset mocks for next test
        websocket1.send_json.reset_mock()
        websocket2.send_json.reset_mock()

        # Disconnect one
        manager.disconnect(websocket1)

        assert len(manager.active_connections) == 1
        assert websocket1 not in manager.active_connections
        assert websocket2 in manager.active_connections

        # Broadcast again
        await manager.broadcast(test_message)

        # Only second websocket should receive
        websocket1.send_json.assert_not_called()
        websocket2.send_json.assert_called_once()

    @pytest.mark.asyncio
    async def test_broadcast_with_failed_send(self):
        """Test broadcasting with a failed send that should disconnect the client."""
        manager = NekoWsNotifier()

        # Create WebSockets - one that will fail on send
        websocket1 = AsyncMock(spec=WebSocket)
        websocket2 = AsyncMock(spec=WebSocket)
        websocket2.send_json.side_effect = WebSocketDisconnect()

        # Connect them
        await manager.connect(websocket1)
        await manager.connect(websocket2)

        # Broadcast should handle the exception and disconnect the failing client
        await manager.broadcast({"type": "test"})

        # Second websocket should be disconnected
        assert len(manager.active_connections) == 1
        assert websocket1 in manager.active_connections
        assert websocket2 not in manager.active_connections


class TestNekoConf:
    """Tests for the NekoConf class."""

    def test_init(self, config_manager):
        """Test initializing the NekoConf."""
        server = NekoConfigServer(config_manager)

        assert server.config == config_manager

        assert hasattr(server, "app")
        # Updated to reflect actual implementation:
        assert hasattr(server, "ws_manager")

    def test_api_get_endpoints(self, test_client, sample_config):
        """Test the GET endpoints for configuration."""
        # Test full config
        response = test_client.get("/api/config")
        assert response.status_code == 200
        assert response.json() == sample_config

        # Test section
        response = test_client.get("/api/config/server")
        assert response.status_code == 200
        assert response.json() == sample_config["server"]

        # Test specific value
        response = test_client.get("/api/config/server/host")
        assert response.status_code == 200
        assert response.json() == sample_config["server"]["host"]

        # Test nonexistent key
        response = test_client.get("/api/config/nonexistent")
        assert response.status_code == 404
        # The exact error format depends on implementation
        assert response.json().get("detail") or response.json().get("error")

    def test_api_set_and_update(self, test_client, config_manager):
        """Test setting and updating config values."""
        # Test setting a single value
        response = test_client.post("/api/config/server/host", json={"value": "127.0.0.1"})
        assert response.status_code == 200
        assert config_manager.get("server.host") == "127.0.0.1"

        # Test bad request - exact behavior depends on implementation
        response = test_client.post("/api/config/server/host", json="not-a-json")
        assert response.status_code in [400, 422]  # FastAPI validation might return 422

        # Test updating multiple values
        update_data = {"server": {"host": "example.com", "port": 9000}}
        response = test_client.post("/api/config", json=update_data)
        assert response.status_code == 200
        # The success key might not be present in all implementations
        assert config_manager.get("server.host") == "example.com"
        assert config_manager.get("server.port") == 9000

    def test_api_delete_and_reload(self, test_client, config_manager):
        """Test deleting values and reloading config."""
        # Test delete
        response = test_client.delete("/api/config/server/debug")
        assert response.status_code == 200
        # Response format depends on implementation
        assert config_manager.get("server.debug") is None

        # Test delete nonexistent key
        response = test_client.delete("/api/config/nonexistent")
        assert response.status_code == 404
        # The error format depends on implementation

        # Test reload
        # Change in-memory config
        config_manager.data["server"]["host"] = "changed_value"

        # Reload via API
        response = test_client.post("/api/config/reload")
        assert response.status_code == 200

        # reload resets to the file value
        assert config_manager.get("server.host") == "localhost"

    @patch("uvicorn.run")
    def test_run(self, mock_run, web_server):
        """Test the run method."""
        web_server.run(host="127.0.0.1", port=9000, reload=True)
        mock_run.assert_called_once()
        args, kwargs = mock_run.call_args
        assert kwargs["host"] == "127.0.0.1"
        assert kwargs["port"] == 9000
        assert kwargs["reload"] is True

    # Skip the websocket tests that rely on specific implementation details
    # which may vary across versions of the application
